package com.simuwang.daq.components.report.parser;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.ReflectUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.mapper.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dos.EmailFieldMappingDO;
import com.simuwang.base.pojo.dto.report.BaseReportDTO;
import com.simuwang.base.pojo.dto.report.ReportBaseInfoDTO;
import com.simuwang.base.pojo.dto.report.ReportData;
import com.simuwang.base.pojo.dto.report.ReportParserParams;
import com.simuwang.daq.components.ReportParseUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;
import java.util.Map;

/**
 * @author wangzaijun
 * @date 2024/9/30 18:13
 * @description 非python接口的报告解析抽象（主要是支持pdf、word和excel等格式）
 */
public abstract class AbstractReportParser<T extends ReportData> implements ReportParser<T> {
    protected final Logger logger = LoggerFactory.getLogger(this.getClass());

    private final EmailFieldMappingMapper fieldMappingMapper;
    /**
     * 字段匹配规则
     */
    protected Map<String, String> fieldMapper;

    public AbstractReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        this.fieldMappingMapper = fieldMappingMapper;
        this.fieldMapper = MapUtil.newHashMap(128);
    }

    /**
     * 初始化数据的方法
     */
    protected void init() {
        List<EmailFieldMappingDO> emailFieldMapping = this.fieldMappingMapper.getEmailFieldMapping(3);
        if (CollUtil.isEmpty(emailFieldMapping)) {
            this.logger.error("未设置报告解析规则！");
            return;
        }
        for (EmailFieldMappingDO mapping : emailFieldMapping) {
            String code = mapping.getCode();
            List<String> names = StrUtil.split(mapping.getName(), ",");
            for (String name : names) {
                this.fieldMapper.putIfAbsent(name, code);
            }
        }
    }

    /**
     * 数据清洗，默认啥也不做
     *
     * @param reportData 结果数据
     */
    protected abstract void cleaningReportData(T reportData);

    /**
     * 构建只有两列表格的dto数据对象
     *
     * @param <DTO>   泛型对象
     * @param fileId  文件id
     * @param clazz   泛型对象
     * @param infoMap 表格转换的函数
     * @return /
     */
    protected <DTO extends BaseReportDTO<?>> DTO buildDto(Integer fileId, Class<DTO> clazz, Map<String, Object> infoMap) {
        try {
            DTO dto = clazz.getDeclaredConstructor().newInstance();
            dto.setFileId(fileId);
            this.buildInfo(infoMap, dto);
            return dto;
        } catch (Exception ignored) {
        }
        return null;
    }

    /**
     * 对象字段设置
     *
     * @param extInfoMap 名称与值的对应关系
     * @param info       待设置的对象
     */
    protected void buildInfo(Map<String, Object> extInfoMap, Object info) {
        if (MapUtil.isEmpty(extInfoMap)) {
            return;
        }
        for (Map.Entry<String, Object> entry : extInfoMap.entrySet()) {
            String k = ReportParseUtils.cleaningValue(entry.getKey());
            String fieldValue = ReportParseUtils.cleaningValue(entry.getValue());
            String fieldName = this.fieldMapper.get(k);
            if (StrUtil.isBlank(fieldName)) {
                continue;
            }
            try {
                ReflectUtil.setFieldValue(info, fieldName, fieldValue);
            } catch (Exception e) {
                this.logger.warn("{} 字段值设置错误：{}", fieldName, e.getMessage());
            }
        }
    }

    /**
     * 构建报告基本信息
     *
     * @param params /
     * @return /
     */
    protected ReportBaseInfoDTO buildReportInfo(ReportParserParams params) {
        Integer fileId = params.getFileId();
        String reportName = params.getFilename();
        ReportBaseInfoDTO reportInfo = new ReportBaseInfoDTO(fileId);
        reportInfo.setReportName(reportName);
        reportInfo.setReportType(ReportParseUtils.matchReportType(reportName));
        reportInfo.setReportDate(ReportParseUtils.matchReportDate(reportName));
        return reportInfo;
    }
}
