package com.simuwang.daq.components.report.parser.pdf;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.exceptions.ExceptionUtil;
import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import com.simuwang.base.common.conts.Constants;
import com.simuwang.base.common.exception.ReportParseException;
import com.simuwang.base.mapper.EmailFieldMappingMapper;
import com.simuwang.base.pojo.dto.report.*;
import com.simuwang.daq.components.CustomPDFTextStripper;
import com.simuwang.daq.components.ReportParseUtils;
import com.simuwang.daq.components.report.parser.AbstractReportParser;
import org.apache.pdfbox.Loader;
import org.apache.pdfbox.io.RandomAccessReadBufferedFile;
import org.apache.pdfbox.pdmodel.PDDocument;
import technology.tabula.CustomObjectExtractor;
import technology.tabula.Page;
import technology.tabula.PageIterator;
import technology.tabula.Table;
import technology.tabula.extractors.SpreadsheetExtractionAlgorithm;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;

/**
 * @author wangzaijun
 * @date 2024/9/29 16:45
 * @description pdf格式的报告解析抽象类
 */
public abstract class AbstractPDReportParser<T extends ReportData> extends AbstractReportParser<T> {
    /**
     * 基金信息表格
     */
    protected Table fundInfoTable;
    /**
     * 去除了水印的所有文本内容
     */
    protected List<String> textList;

    public AbstractPDReportParser(EmailFieldMappingMapper fieldMappingMapper) {
        super(fieldMappingMapper);
    }

    @Override
    public T parse(ReportParserParams params) throws IOException, ReportParseException {
        // 先初始化为null
        this.fundInfoTable = null;
        this.textList = null;
        // 初始化
        this.init();
        String filename = params.getFilename();
        // 解析报告和表格
        try (PDDocument document = Loader.loadPDF(new RandomAccessReadBufferedFile(params.getFilepath()))) {
            // 识别所有文字（去水印后的）
            CustomPDFTextStripper stripper = new CustomPDFTextStripper();
            stripper.setSortByPosition(true);
            String text = stripper.getText(document).replace(Constants.WATERMARK_REPLACE, StrUtil.EMPTY);
            this.textList = StrUtil.split(text, System.lineSeparator());
            this.textList.removeIf(StrUtil::isBlank);
            if (this.textList.isEmpty()) {
                throw new ReportParseException(ReportParseStatus.REPORT_IS_SCAN, filename);
            }
            // 解析所有表格（单元格字符去水印）
            List<Table> tables = ListUtil.list(true);
            SpreadsheetExtractionAlgorithm extractionAlgorithm = new SpreadsheetExtractionAlgorithm();
            // 自定义表格提取工具，去除单元格中的水印文字
            PageIterator pageIterator = new CustomObjectExtractor(document).extract();
            while (pageIterator.hasNext()) {
                Page page = pageIterator.next();
                tables.addAll(extractionAlgorithm.extract(page));
            }
            if (tables.isEmpty()) {
                throw new ReportParseException(ReportParseStatus.REPORT_IS_SCAN, filename);
            }
            this.initTableInfo(tables);
        }
        try {
            // 报告基本信息
            ReportBaseInfoDTO reportInfo = this.buildReportInfo(params);
            // 解析报告中主体基金的基本信息
            ReportFundInfoDTO reportFundInfo = this.buildFundInfo(params);
            // 解析其他表格信息并且设置结果字段
            T reportData = this.parseExtInfoAndSetData(reportInfo, reportFundInfo);
            // 数据清洗后返回
            this.cleaningReportData(reportData);
            return reportData;
        } catch (ReportParseException e) {
            throw e;
        } catch (Exception e) {
            this.logger.warn("报告解析错误：{}", ExceptionUtil.stacktraceToString(e));
            throw new ReportParseException(ReportParseStatus.NOT_A_FIXED_FORMAT, filename);
        }
    }

    /**
     * 初始化解析所有表格数据
     *
     * @param tables 按固定的表格模式划分到不同的对象中
     */
    protected abstract void initTableInfo(List<Table> tables);

    /**
     * 绑定基金基本信息（年报的基金基本信息解析逻辑要覆盖重写）
     *
     * @param params /
     * @return /
     */
    protected ReportFundInfoDTO buildFundInfo(ReportParserParams params) {
        Table fundInfoTable = this.fundInfoTable;
        if (fundInfoTable == null) {
            throw new ReportParseException(ReportParseStatus.PARSE_FUND_INFO_FAIL, params.getFilename());
        }
        // 基金基本信息映射
        Map<String, Object> extInfoMap = this.parseFundInfo(fundInfoTable);
        return this.buildDto(params.getFileId(), ReportFundInfoDTO.class, extInfoMap);
    }

    /**
     * 解析基金基本信息表格
     *
     * @param fundInfoTable 表格
     * @return /
     */
    protected abstract Map<String, Object> parseFundInfo(Table fundInfoTable);

    /**
     * 解析报告的其他信息并设置到对象中
     *
     * @param reportInfo 报告基本信息
     * @param fundInfo   报告中基金基本信息
     * @return /
     */
    protected abstract T parseExtInfoAndSetData(ReportBaseInfoDTO reportInfo, ReportFundInfoDTO fundInfo);

    @Override
    protected void cleaningReportData(T reportData) {
        // cleaning.
    }

    /**
     * 构建只有两列表格的dto数据对象，如果有分级基金时（并且一个表格可能跨页）
     *
     * @param <DTO>    泛型对象
     * @param fileId   文件id
     * @param tables   表格
     * @param clazz    泛型对象
     * @param function 表格转换的函数
     * @return /
     */
    protected <DTO extends BaseReportLevelDTO<?>> List<DTO> buildLevelDto(Integer fileId, List<Table> tables, Class<DTO> clazz,
                                                                          Function<Table, Map<String, Object>> function) {
        List<DTO> dtos = ListUtil.list(true);
        // 信息表格字段和值映射
        List<Map<String, Object>> infos = ListUtil.list(true);
        Map<String, Object> infoMap = null;
        for (Table table : tables) {
            Map<String, Object> temp = function.apply(table);
            for (String key : temp.keySet()) {
                // 如果infoMap为null，先声明然后放在infos中
                if (infoMap == null) {
                    infoMap = MapUtil.newHashMap(16);
                    infos.add(infoMap);
                }
                // 如果infoMap中包含了该key时，先放infos中然后重新声明新map对象
                if (infoMap.containsKey(key)) {
                    infos.add(new HashMap<>(infoMap));
                    infoMap = MapUtil.newHashMap(16);
                } else {
                    infoMap.put(key, temp.get(key));
                }
            }
        }
        // 分级基金匹配
        List<String> levels = ReportParseUtils.matchTieredFund(String.join(",", this.textList));
        for (int i = 0; i < infos.size(); i++) {
            DTO dto = this.buildDto(fileId, clazz, infos.get(i));
            if (dto == null) {
                continue;
            }
            dto.setLevel(levels.get(i));
            dtos.add(dto);
        }
        return dtos;
    }
}
